import Foundation

// Model & JSON

struct Task {
    let name: String
    let completed: Bool
    let position: Int
    
    static func init_(name: String, completed: Bool, position: Int) -> Task {
        return Task(name: name, completed: completed, position: position)
    }
}

let json = ["name": "Do this", "completed": false, "position": 10]

// Functional operators

infix operator >>> { associativity left precedence 150 }

func >>> <A, B>(a: A?, f: A -> B?) -> B? {
    if let a = a {
        return f(a)
    } else {
        return nil
    }
}

// Casting helpers

func autocast<T>(x: AnyObject) -> T? {
    return x as? T
}

// Optionals helpers

func ensure<A, B, C>(a: A?, b: B?, c: C?) -> (A, B, C)? {
    if let a = a, b = b, c = c {
        return (a, b, c)
    } else {
        return nil
    }
}

// JSON parsing helpers

func decodeJSON<A, B, C, R>(json: [String: AnyObject], f: (A, B, C) -> R, a: String, b: String, c: String) -> R? {
    let a = json[a] as? A
    let b = json[b] as? B
    let c = json[c] as? C
    let params = ensure(a, b, c)
    return params.map(f)
}

// Decoding

let task = decodeJSON(json, Task.init_, "name", "completed", "position")
